<?php

namespace WAAVE\Complete\Controller\Checkout;

use Magento\Framework\App\Action\Action;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Sales\Model\OrderFactory;
use Magento\Sales\Model\Order;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
use WAAVE\Complete\Gateway\Config\Checkout\Config;
use WAAVE\Compliance\Model\ConfigurationFactory;

class Index extends Action
{
    /**
     * @var Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var Magento\Checkout\Model\Session
     */
    private $checkoutSession;

    /**
     * @var Magento\Sales\Model\OrderFactory
     */
    private $orderFactory;

    /**
     * @var WAAVE\Compliance\Model\ConfigurationFactory
     */
    private $configurationFactory;

    /**
     * @var WAAVE\Complete\Gateway\Config\Checkout\Config
     */
    private $gatewayConfig;

    /**
     * @var Psr\Log\LoggerInterface
     */
    private $logger;

    public function __construct(
        Config $gatewayConfig,
        Session $checkoutSession,
        StoreManagerInterface $storeManager,
        OrderFactory $orderFactory,
        ConfigurationFactory $configurationFactory,
        LoggerInterface $logger,
        Context $context
    ) {
        $this->checkoutSession      = $checkoutSession;
        $this->orderFactory         = $orderFactory;
        $this->configurationFactory = $configurationFactory;
        $this->gatewayConfig        = $gatewayConfig;
        $this->storeManager         = $storeManager;

        parent::__construct($context);
    }

    /**
     * Get configuration factory
     */
    protected function getConfiguration()
    {
        return $this->configurationFactory->create()->getCollection()->getFirstItem();
    }

    /**
     * Get order
     */
    protected function getOrder()
    {
        $orderId = $this->checkoutSession->getLastRealOrderId();

        if (!isset($orderId)) {
            return null;
        }

        $order = $this->orderFactory->create()->loadByIncrementId($orderId);

        if (!$order->getId()) {
            return null;
        }

        return $order;
    }

    /**
     * Post to checkout
     */
    protected function postToCheckout($gatewayUrl, $payload)
    {
        echo
        "<html>
            <body>
            <form id='form' action='$gatewayUrl' method='GET'>";
        foreach ($payload as $key => $value) {
            echo "<input type='hidden' id='$key' name='$key' value='".htmlspecialchars($value, ENT_QUOTES)."'/>";
        }
        echo
            '</form>
            </body>';
        echo
            '<script>
                var form = document.getElementById("form");
                form.submit();
            </script>
        </html>';
    }

    /**
     * Execute
     */
    public function execute()
    {
        $order = $this->getOrder();

        if ($order->getState() != Order::STATE_NEW) {
            $this->_redirect('checkout/onepage/failure', ['_secure'=> false]);
            return;
        }

        $baseUrl = $this->storeManager->getStore()->getBaseUrl();

        $returnUrl   = $baseUrl . 'checkout/onepage/success';
        $cancelUrl   = $baseUrl . 'waave_complete/checkout/cancel';
        $callbackUrl = $baseUrl . 'rest/V1/waave_complete/callback';

        $venueId = $this->getConfiguration()->getData('venue_id');

        $payload = array(
            'return_url'    => $returnUrl,
            'cancel_url'    => $cancelUrl,
            'callback_url'  => $callbackUrl,
            'amount'        => $order->getTotalDue(),
            'currency'      => $order->getOrderCurrencyCode(),
            'reference_id'  => $order->getRealOrderId(),
            'access_key'    => $this->getConfiguration()->getAccessKey(),
            'private_key'   => $this->getConfiguration()->getPrivateKey(),
            'venue_id'      => $venueId
        );

        $gatewayUrl  = $this->gatewayConfig->getGatewayURL();
        $this->postToCheckout($gatewayUrl, $payload);
    }

}
