<?php

namespace WAAVE\Complete\Controller\Validate;

use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Checkout\Model\Session;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Model\MaskedQuoteIdToQuoteIdInterface;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
use WAAVE\Complete\Gateway\Config\Checkout\Config;
use WAAVE\Compliance\Model\ConfigurationFactory;
use WAAVE\Complete\Helper\Data as DataHelper;

class Index extends Action
{
    /**
     * @var WAAVE\Compliance\Model\ConfigurationFactory
     */
    private $configurationFactory;

    /**
     * @var JsonFactory
     */
    private $resultJsonFactory;

    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;

    /**
     * @var MaskedQuoteIdToQuoteIdInterface
     */
    private $maskedQuoteIdToQuoteId;

    /**
     * @var WAAVE\Complete\Gateway\Config\Checkout\Config
     */
    private $gatewayConfig;

    /**
     * @var Magento\Checkout\Model\Session
     */
    private $checkoutSession;

    /**
     * @var DataHelper
     */
    private $helper;

    /**
     * @var Psr\Log\LoggerInterface
     */
    private $logger;

    public function __construct(
        Config $gatewayConfig,
        Session $checkoutSession,
        ConfigurationFactory $configurationFactory,
        JsonFactory $resultJsonFactory,
        MaskedQuoteIdToQuoteIdInterface $maskedQuoteIdToQuoteId,
        CartRepositoryInterface $cartRepository,
        LoggerInterface $logger,
        DataHelper $helper,
        Context $context
    ) {
        $this->checkoutSession        = $checkoutSession;
        $this->configurationFactory   = $configurationFactory;
        $this->maskedQuoteIdToQuoteId = $maskedQuoteIdToQuoteId;
        $this->gatewayConfig          = $gatewayConfig;
        $this->resultJsonFactory      = $resultJsonFactory;
        $this->cartRepository         = $cartRepository;
        $this->helper                 = $helper;
        $this->logger                 = $logger;

        parent::__construct($context);
    }

    /**
     * Get configuration factory
     */
    protected function getConfiguration()
    {
        return $this->configurationFactory->create()->getCollection()->getFirstItem();
    }

    /**
     * Execute
     */
    public function execute()
    {
        $loggedIn = $this->getRequest()->getParam('loggedIn');
        $quoteId  = $this->getRequest()->getParam('quoteId');

        if ('true' == $loggedIn) {
            $cart = $this->cartRepository->get($quoteId);
        } else {
            $quote = $this->maskedQuoteIdToQuoteId->execute($quoteId);
            $cart  = $this->cartRepository->get($quote);
        }

        $billing = $cart->getBillingAddress();
        $billingArr = [
            'billing_country'   => $billing->getCountryId(),
            'billing_address_1' => $billing->getStreetLine(1),
            'billing_address_2' => '',
            'billing_city'      => $billing->getCity(),
            'billing_state'     => $this->helper->getStateCode($billing->getRegion()),
            'billing_postcode'  => $billing->getPostcode()
        ];

        $shipping = $cart->getShippingAddress();
        $shippingArr = [
            'shipping_country'   => $shipping->getCountryId(),
            'shipping_address_1' => $shipping->getStreetLine(1),
            'shipping_address_2' => '',
            'shipping_city'      => $shipping->getCity(),
            'shipping_state'     => $this->helper->getStateCode($shipping->getRegion()),
            'shipping_postcode'  => $shipping->getPostcode()
        ];

        $products = [];

        $items = $cart->getItems();
        foreach ($items as $item) {
            $products[] = [
                'id'         => $item['product_id'],
                'name'       => $item['product']['name'],
                'sku'        => $item['product']['sku'],
                'price'      => $item['product']['price'],
                'categories' => $item->getProduct()->getCategoryIds()
            ];
        }

        $venueId = $this->getConfiguration()->getData('venue_id');

        $email = $this->getRequest()->getParam('shippingEmail');
        if (empty($email)) {
            $email = $this->getRequest()->getParam('guestEmail');
        }

        $params = [
            'venue_id' => $venueId,
            'email'    => $email,
            'amount'   => $cart->getGrandTotal(),
            'products' => $products,
            'shipping' => $shippingArr,
            'billing'  => $billingArr
        ];

        $url = $this->gatewayConfig->getValidateURL();
        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL             => $url,
            CURLOPT_RETURNTRANSFER  => true,
            CURLOPT_POSTFIELDS      => json_encode($params),
            CURLOPT_VERBOSE         => true,
            CURLOPT_TIMEOUT         => 30,
            CURLOPT_SSL_VERIFYPEER  => false,
            CURLOPT_HTTPHEADER      => [
                "Content-Type: application/json"
            ]
        ]);

        $response = curl_exec($curl);

        curl_close($curl);

        $resultJson = $this->resultJsonFactory->create();

        $body = json_decode($response, true);
        if (!empty($body['message'])) {
            return $resultJson->setData(['message' => $body['message']]);
        }

        $this->checkoutSession->setValidationCode($body['validation_code']);

        return $resultJson->setData(['success' => true]);
    }

}
