<?php

namespace WAAVE\Complete\Model;

use Magento\Sales\Model\Order;
use Magento\Sales\Model\OrderFactory;
use Psr\Log\LoggerInterface as PsrLogger;

class OrderManagement
{

    /**
     * @var PsrLogger
     */
    protected $logger;

    /**
     * @var OrderFactory
     */
    protected $orderFactory;

    public function __construct(OrderFactory $orderFactory, PsrLogger $logger)
    {
        $this->orderFactory = $orderFactory;
        $this->logger = $logger;
    }

    /**
    * {@inheritdoc}
    */
    public function updateOrder($reference_id, $amount, $status)
    {
        $order = $this->orderFactory->create()->loadByIncrementId($reference_id);

        // Validate
        if (!$order->getId()) {
            $this->logger->debug("WAAVE: the order is invalid");
            return 'FAILED-MAGE2: the reference_id is invalid';
        }
        if ($order->getState() === Order::STATE_PROCESSING) {
            $this->logger->debug("WAAVE: the order status is processing");
            return 'FAILED-MAGE2: the order is processing';
        }

        if ($status != "completed") {
            $this->logger->debug("WAAVE: the gateway status is invalid");
            return 'FAILED-MAGE2: the gateway status is not be completed';
        }

        if ($order->getTotalDue() != $amount) {
            $this->logger->debug("WAAVE: the total amount is invalid");
            return 'FAILED-MAGE2: the total amount is invalid';
        }

        $orderState = Order::STATE_PROCESSING;
        $orderStatus = $order->getConfig()->getStateDefaultStatus($orderState);

        $order->setState($orderState)
            ->setStatus($orderStatus)
            ->addStatusHistoryComment("Your payment with WAAVE is complete.");

        $order->save();

        return 'DONE-MAGE2';
    }
}