<?php

/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace WAAVE\Complete\Model\Ui\Direct;

use WAAVE\Complete\Gateway\Config\Direct\Config;
use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Framework\View\Asset\Repository;

/**
 * Class ConfigProvider
 */
final class ConfigProvider implements ConfigProviderInterface
{
    const CODE = 'waave_direct';

    private $assetRepository;

    private $gatewayConfig;

    public function __construct(Repository $assetRepository, Config $gatewayConfig)
    {
        $this->assetRepository = $assetRepository;
        $this->gatewayConfig   = $gatewayConfig;
    }

    /**
     * Retrieve assoc array of checkout configuration
     *
     * @return array
     */
    public function getConfig()
    {
        $logo = $this->assetRepository->getUrlWithParams('WAAVE_Complete::images/logo.png', []);
        $cvv  = $this->assetRepository->getUrlWithParams('WAAVE_Complete::images/cvv.png', []);
        $vgs  = $this->gatewayConfig->getVgsData();

        return [
            'payment' => [
                self::CODE => [
                    'logo'    => $logo,
                    'vgs_env' => $vgs['env'],
                    'vgs_id'  => $vgs['id'],
                ],
                'ccform' => [
                    'months' => [
                        self::CODE => [
                            1 => 1,
                            2 => 2,
                            3 => 3,
                            4 => 4,
                            5 => 5,
                            6 => 6,
                            7 => 7,
                            8 => 8,
                            9 => 9,
                            10 => 10,
                            11 => 11,
                            12 => 12,
                        ]
                    ],
                    'years' => [
                        self::CODE => [
                            2022 => 2022,
                            2023 => 2023,
                            2024 => 2024,
                            2025 => 2025,
                            2026 => 2026,
                            2027 => 2027,
                            2028 => 2028,
                            2029 => 2029,
                            2030 => 2030,
                        ]
                    ],
                    'availableTypes' => [
                        self::CODE => [
                            'VI'  => 'Visa',
                            'MC'  => 'MasterCard',
                            'AE'  => 'American Express',
                            'JCB' => 'JCB'
                        ]
                    ],
                    'cvvImageUrl' => [
                        self::CODE => $cvv
                    ]
                ]
            ]
        ];
    }
}
