/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
/*browser:true*/
/*global define*/
define(
    [
        'jquery',
        'Magento_Payment/js/view/payment/cc-form',
        'Magento_Checkout/js/model/quote',
        'Magento_Customer/js/model/customer',
        'Magento_Ui/js/model/messageList',
        'mage/translate',
        'mage/url',
        'Magento_Checkout/js/model/full-screen-loader',
        'Magento_Payment/js/model/credit-card-validation/validator'
    ],
    function ($, Component, quote, customer, messageList, $t, url, fullScreenLoader) {
        'use strict';

        return Component.extend({
            defaults: {
                template: 'WAAVE_Complete/payment/form_direct',
                redirectAfterPlaceOrder: false,
                vgsForm: null,
                token: '',
            },

            getCode: function() {
                return 'waave_direct';
            },

            isActive: function() {
                return true;
            },

            hasVerification: function() {
                return true;
            },

            initialize: function() {
                var self = this;

                this._super();

                self.vgsForm = VGSCollect.create(
                    window.checkoutConfig.payment.waave_direct.vgs_id,
                    window.checkoutConfig.payment.waave_direct.vgs_env,
                    (state) => {},
                );

                setTimeout(function() {
                    self.vgsForm.field('#cc-number', {
                        type: 'card-number',
                        name: 'card_number',
                        successColor: '#4F8A10',
                        errorColor: '#D8000C',
                        placeholder: 'Credit Card Number',
                        showCardIcon: true,
                        validations: ['required', 'validCardNumber'],
                    });

                    self.vgsForm.field('#cc-expiration-date', {
                        type: 'card-expiration-date',
                        name: 'card_exp',
                        successColor: '#4F8A10',
                        errorColor: '#D8000C',
                        placeholder: 'MM / YY',
                        validations: ['required', 'validCardExpirationDate'],
                    });

                    self.vgsForm.field('#cc-cvc', {
                        type: 'card-security-code',
                        name: 'cvv',
                        successColor: '#4F8A10',
                        errorColor: '#D8000C',
                        placeholder: 'CVC',
                        validations: ['required', 'validCardSecurityCode'],
                    });
                }, 2000);
            },
            placeOrderDirectClick: function() {
                const self = this;

                if (self.vgsForm === null) {
                    return false;
                }

                fullScreenLoader.startLoader();

                self.vgsForm.submit(
                    "/api/v1/cards/vgs",
                    {},
                    (status, response) => {
                        if (status === 200) {
                            self.token = response.card_token;

                            $.ajax({
                                url: url.build('waave_complete/validate/index'),
                                type: 'POST',
                                data: {
                                    quoteId: quote.getQuoteId(),
                                    guestEmail: quote.guestEmail,
                                    loggedIn: customer.isLoggedIn,
                                    shippingEmail: quote.shippingAddress().email,
                                },
                                success: function(response) {
                                    if (response.message) {
                                        messageList.addErrorMessage({ message: response.message });
                                        fullScreenLoader.stopLoader();

                                        return false;
                                    }

                                    self.placeOrder();
                                }
                            });
                        } else {
                            messageList.addErrorMessage({ message: response.message });
                            fullScreenLoader.stopLoader();

                            return false;
                        }
                    },
                    (errors) => {
                        messageList.addErrorMessage({ message: $t('Please check your credit card information again') });
                        fullScreenLoader.stopLoader();

                        return false;
                    },
                );
            },

            getLogo: function() {
                return window.checkoutConfig.payment.waave_direct.logo;
            },

            afterPlaceOrder: function() {
                window.location.replace(url.build('waave_complete/direct/index') + '?token=' + this.token);
            }
        });
    }
);