<?php

namespace WAAVE\Compliance\Block\Adminhtml;

use Magento\Backend\Block\Template\Context;
use WAAVE\Compliance\Model\ConfigurationFactory;

class Configuration extends \Magento\Backend\Block\Template
{
    protected $_configurationFactory;

    /**
     * @param \Magento\Backend\Block\Template\Context $context
     */
    public function __construct(
        Context $context,
        ConfigurationFactory $configurationFactory
    ) {
        parent::__construct($context);
        $this->_configurationFactory = $configurationFactory;
    }

    /**
     * Prepares layout
     *
     * @return \Magento\Framework\View\Element\AbstractBlock
     */
    protected function _prepareLayout()
    {
        $this->getToolbar()->addChild(
            'save_button',
            \Magento\Backend\Block\Widget\Button::class,
            [
                'label' => __('Save Config'),
                'class' => 'save primary',
                'data_attribute' => [
                    'mage-init' => ['button' => ['event' => 'save', 'target' => '#configuration-form']],
                ]
            ]
        );

        return parent::_prepareLayout();
    }

    /**
     * Returns URL for save action
     *
     * @return string
     * @codeCoverageIgnore
     */
    public function getFormActionUrl()
    {
        return $this->getUrl('/*/save');
    }

    /**
     * Returns access key for save action
     *
     * @return string
     * @codeCoverageIgnore
     */
    public function getAccessKey()
    {
        $configuration = $this->getConfiguration();
        return $configuration->getData('access_key');
    }

    /**
     * Returns private key for save action
     *
     * @return string
     * @codeCoverageIgnore
     */
    public function getPrivateKey()
    {
        $configuration = $this->getConfiguration();
        return $configuration->getData('private_key');
    }

    /**
     * Returns Venue ID for save action
     *
     * @return string
     * @codeCoverageIgnore
     */
    public function getVenueID()
    {
        $configuration = $this->getConfiguration();
        return $configuration->getData('venue_id');
    }

    /**
     * Returns password for save action
     *
     * @return string
     * @codeCoverageIgnore
     */
    public function getPassword()
    {
        $configuration = $this->getConfiguration();
        return $configuration->getData('password');
    }

    /**
     * Get test mode
     * 
     * @return string
     * @codeCoverageIgnore
     */
    public function getTestMode()
    {
        $configuration = $this->getConfiguration();
        return $configuration->getData('test_mode');
    }

    /**
     * Returns configuration
     *
     * @return object
     * @codeCoverageIgnore
     */
    protected function getConfiguration()
    {
        $collection = $this->_configurationFactory->create()->getCollection();
        return $collection->getFirstItem();
    }
}