<?php

namespace WAAVE\Compliance\Controller\Adminhtml\Configuration;

use Magento\Framework\App\Action\HttpPostActionInterface as HttpPostActionInterface;
use Magento\Backend\App\Action\Context;
use WAAVE\Compliance\Controller\Adminhtml\Configuration;
use WAAVE\Compliance\Model\ConfigurationFactory;

class Save extends Configuration implements HttpPostActionInterface
{
    const PROD_URL = 'https://getwaave.co/compliance/active';
    const STAG_URL = 'https://staging.getwaave.co/compliance/active';

    protected $_configurationFactory;

    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \WAAVE\Compliance\Model\ConfigurationFactory
     */
    public function __construct(
        Context $context,
        ConfigurationFactory $configurationFactory
    ) {
        parent::__construct($context);
        $this->_configurationFactory = $configurationFactory;
    }

    /**
     * Save configuration
     *
     * @return ResultInterface
     */
    public function execute()
    {
        /** @var Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        $accessKey  = $this->getRequest()->getParam('access_key', null);
        $privateKey = $this->getRequest()->getParam('private_key', null);
        $venueId    = $this->getRequest()->getParam('venue_id', null);
        $password   = $this->getRequest()->getParam('password', null);
        $testMode   = $this->getRequest()->getParam('test_mode', 1);

        $collection = $this->_configurationFactory->create()->getCollection();
        $configuration = $collection->getFirstItem();

        $configuration->setAccessKey($accessKey);
        $configuration->setPrivateKey($privateKey);
        $configuration->setVenueId($venueId);
        $configuration->setPassword($password);
        $configuration->setTestMode($testMode);
        $configuration->save();

        try {
            $versionApi  = $this->_url->getBaseUrl() . 'rest/V1/waave-compliance/version';
            $productApi  = $this->_url->getBaseUrl() . 'rest/V1/waave-compliance/products';
            $categoryApi = $this->_url->getBaseUrl() . 'rest/V1/waave-compliance/categories';

            $params = array(
                'version'       => '2.0.1',
                'venue_id'      => $configuration->getVenueId(),
                'password'      => $configuration->getPassword(),
                'ping_url'      => $versionApi,
                'category_url'  => $categoryApi,
                'menu_item_url' => $productApi
            );

            $url = self::PROD_URL;
            if ($testMode) {
                $url = self::STAG_URL;
            }

            $client = new \GuzzleHttp\Client();
            $client->request('POST', $url, [
                'form_params' => $params
            ]);

            $this->messageManager->addSuccessMessage(__('Your configurations saved successfully.'));

        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        }

        return $resultRedirect->setPath('waave_compliance/*/');
    }
}