<?php

namespace WAAVE\Compliance\Model;

use Magento\Catalog\Api\ProductRepositoryInterface;

class ProductManagement
{
    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     */
    public function __construct(
        ProductRepositoryInterface $productRepository
    ) {
        $this->productRepository = $productRepository;
    }

    /**
     * @inheritdoc
     */
    public function getList($after=null)
    {
        $searchCriteria = new \Magento\Framework\Api\SearchCriteria();
        $searchCriteria->setData('current_page', 1);
        //$searchCriteria->setData('page_size', 1);

        $filterVisibility = new \Magento\Framework\Api\Filter();
        $filterVisibility->setField('visibility');
        $filterVisibility->setValue(4);

        $group = new \Magento\Framework\Api\Search\FilterGroup();
        $group->setFilters([$filterVisibility]);

        $groups = [$group];

        if ($after) {
            $filterAfter = new \Magento\Framework\Api\Filter();
            $filterAfter->setField('updated_at');
            $filterAfter->setValue($after);
            $filterAfter->setConditionType('gt');

            $group = new \Magento\Framework\Api\Search\FilterGroup();
            $group->setFilters([$filterAfter]);

            $groups[] = $group;
        }

        $searchCriteria->setFilterGroups($groups);
        $list = $this->productRepository->getList($searchCriteria);

        $items = [];
        foreach ($list->getItems() as $value) {
            $categories = [];
            $collection = $value->getCategoryCollection()->addAttributeToSelect('*');
            foreach ($collection as $category) {
                $categories[] = [
                    'id'   => $category->getId(),
                    'name' => $category->getName()
                ];
            }

            $items[] = [
                'id' => $value->getData('entity_id'),
                'name' => $value->getData('name'),
                'sku' => $value->getData('sku'),
                'price' => $value->getData('price'),
                'permalink' => $value->getProductUrl(),
                'categories' => $categories
            ];
        }

        return $items;
    }
}